@@ -82,6 +82,7 @@ GWeatherInfo *           gweather_info_new_for_world       (GWeatherLocation *wo
 							    GWeatherForecastType forecast_type);
 void                     gweather_info_update              (GWeatherInfo *info);
 void			 gweather_info_abort		   (GWeatherInfo *info);
+void                     gweather_info_store_cache         (void);
 
 GWeatherProvider         gweather_info_get_enabled_providers (GWeatherInfo        *info);
 void                     gweather_info_set_enabled_providers (GWeatherInfo        *info,
@@ -117,6 +118,7 @@ gchar * 		 gweather_info_get_temp_summary	   (GWeatherInfo *info);
 gchar *			 gweather_info_get_weather_summary (GWeatherInfo *info);
 
 const gchar *		 gweather_info_get_icon_name	   (GWeatherInfo *info);
+const gchar *		 gweather_info_get_symbolic_icon_name	   (GWeatherInfo *info);
 gint			 gweather_info_next_sun_event	   (GWeatherInfo *info);
 
 gboolean                 gweather_info_is_daytime          (GWeatherInfo *info);
