@@ -29,7 +29,7 @@
 #define GWEATHER_I_KNOW_THIS_IS_UNSTABLE
 #include "weather-priv.h"
 
-/**
+/*
  *  Humans don't deal well with .MONDAY...SUNNY AND BLAH BLAH.TUESDAY...THEN THIS AND THAT.WEDNESDAY...RAINY BLAH BLAH.
  *  This function makes it easier to read.
  */
@@ -176,7 +176,7 @@ parseForecastXml (const char *buff, GWeatherInfo *master_info)
 
                         for (i = 0; i < 7;  i++) {
                             GWeatherInfo *nfo = _gweather_info_new_clone (master_info);
-			    nfo->priv->update = update_times[i];
+			    nfo->priv->current_time = nfo->priv->update = update_times[i];
 
                             if (nfo)
                                 res = g_slist_append (res, nfo);
@@ -367,7 +367,7 @@ iwin_finish (SoupSession *session, SoupMessage *msg, gpointer data)
         /* forecast data is not really interesting anyway ;) */
         g_warning ("Failed to get IWIN forecast data: %d %s\n",
                    msg->status_code, msg->reason_phrase);
-        request_done (info, FALSE);
+        _gweather_info_request_done (info);
         return;
     }
 
@@ -378,11 +378,11 @@ iwin_finish (SoupSession *session, SoupMessage *msg, gpointer data)
     else
         priv->forecast = formatWeatherMsg (g_strdup (msg->response_body->data));
 
-    request_done (info, TRUE);
+    _gweather_info_request_done (info);
 }
 
 /* Get forecast into newly alloc'ed string */
-void
+gboolean
 iwin_start_open (GWeatherInfo *info)
 {
     GWeatherInfoPrivate *priv;
@@ -390,21 +390,28 @@ iwin_start_open (GWeatherInfo *info)
     WeatherLocation *loc;
     SoupMessage *msg;
 
-    g_return_if_fail (info != NULL);
+    g_return_val_if_fail (info != NULL, FALSE);
 
     priv = info->priv;
-    loc = priv->location;
-    g_return_if_fail (loc != NULL);
+    loc = &priv->location;
+    g_return_val_if_fail (loc != NULL, FALSE);
 
-    if ((!loc->zone || loc->zone[0] == '-') && (priv->forecast_type != GWEATHER_FORECAST_LIST || !loc->latlon_valid))
-        return;
+    /* No zone (or -) means no weather information from national offices */
+    if ((!loc->zone || loc->zone[0] == '-'))
+        return FALSE;
 
-    if (priv->forecast) {
-        g_free (priv->forecast);
-        priv->forecast = NULL;
-    }
+    /* Zones starting with : are for the UK Met Office, @ is for Austrialian
+       Bureau of Metereology. GWEATHER_FORECAST_LIST only works for US, so bail
+       out early if the location is outside.
+    */
+    if (priv->forecast_type == GWEATHER_FORECAST_LIST &&
+	(loc->zone[0] == ':' || loc->zone[0] == '@'))
+	return FALSE;
 
-    free_forecast_list (info);    
+    /* We also need a pair of coordinates for GWEATHER_FORECAST_LIST */
+    if (priv->forecast_type == GWEATHER_FORECAST_LIST &&
+	!loc->latlon_valid)
+	return FALSE;
 
     if (priv->forecast_type == GWEATHER_FORECAST_LIST) {
         /* see the description here: http://www.weather.gov/forecasts/xml/ */
@@ -424,17 +431,17 @@ iwin_start_open (GWeatherInfo *info)
 	soup_session_queue_message (priv->session, msg, iwin_finish, info);
 
         priv->requests_pending++;
-        return;
+        return TRUE;
     }
 
     if (loc->zone[0] == ':') {
         /* Met Office Region Names */
         metoffice_start_open (info);
-        return;
+        return TRUE;
     } else if (loc->zone[0] == '@') {
         /* Australian BOM forecasts */
         bom_start_open (info);
-        return;
+        return TRUE;
     }
 
     /* The zone for Pittsburgh (for example) is given as PAZ021 in the locations
@@ -454,4 +461,6 @@ iwin_start_open (GWeatherInfo *info)
     soup_session_queue_message (priv->session, msg, iwin_finish, info);
 
     priv->requests_pending++;
+
+    return TRUE;
 }
